@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_drag.erl,v 1.196 2005/09/15 05:23:06 bjorng Exp $
+%%     $Id: wings_drag.erl 382 2008-09-30 13:11:33Z optigon $
 %%
 
 -module(wings_drag).
@@ -25,12 +25,14 @@
 -record(drag,
 	{x,					%Original 2D position
 	 y,
-	 xs=0,                                  %Summary of mouse movements
+	 xs=0,					%Summary of mouse movements
 	 ys=0,
-	 zs=0,
-	 xt=0,                                  %Last warp length
+	 zs=0,                  %Z move in screen relative
+	 fp=0,                  %An optional forth drag parameter
+	 xt=0,					%Last warp length
 	 yt=0,
 	 mmb_count=0,
+	 fp_count=0,
 	 offset,				%Offset for each dimension.
 	 unit,					%Unit that drag is done in.
 	 unit_sc,				%Scales for each dimension.
@@ -38,7 +40,7 @@
 	 falloff,				%Magnet falloff.
 	 mode_fun,				%Special mode.
 	 mode_data,				%State for mode.
-	 info="",				%Information line.
+	 info="",                               %Information line.
 	 st,					%Saved st record.
 	 last_move				%Last move.
 	}).
@@ -55,8 +57,7 @@ setup(Tvs, Units, Flags, St) ->
     wings_io:grab(),
     wings_wm:grab_focus(),
     Offset0 = proplists:get_value(initial, Flags, []),
-    Offset1 = pad_offsets(Offset0),
-    Offset = setup_offsets(Offset1, Units),
+    Offset = pad_offsets(Offset0),
     UnitSc = unit_scales(Units),
     Falloff = falloff(Units),
     {ModeFun,ModeData} = setup_mode(Flags, Falloff),
@@ -77,16 +78,16 @@ setup(Tvs, Units, Flags, St) ->
     end,
     {drag,Drag}.
 
-setup_mode(Flags, Falloff) ->    
+setup_mode(Flags, Falloff) ->
     case proplists:get_value(mode, Flags, none) of
 	none ->
 	    {standard_mode_fun(Falloff),none};
 	{_,_}=Mode ->
 	    Mode
     end.
-    
+
 standard_mode_fun(Falloff) ->
-    Help0 = ?__(1,"[Shift] and/or [Ctrl] Constrain"),
+    Help0 = ?__(1,"[Shift] and/or [Ctrl] and/or [Alt] Constrain"),
     Help = case Falloff of
 	       none -> Help0;
 	       _ ->
@@ -99,21 +100,64 @@ standard_mode_fun(Falloff) ->
     end.
 
 unit_scales(Units) ->
+    case wings_pref:get_value(drag_custom) of
+      false ->
+        default_drag(Units);
+      true -> 
+        custom_drag(Units)
+    end.
+
+custom_drag(Units) ->
+    BasicSc = 1/500,
+    DistSc = custom_absolute_drag(),
+    PercentSc = custom_relative_drag(),
+    AngleSc = custom_rotations_drag(),
+    unit_scales_1(Units, BasicSc, DistSc, PercentSc, AngleSc).
+
+custom_absolute_drag() ->
+    Speed = wings_pref:get_value(drag_speed_abs),
+    case wings_pref:get_value(drag_cam_dist_abs) of
+      true ->
+        #view{distance=D} = wings_view:current(),
+        D/((11-Speed)*((11-Speed)*300));
+      false ->
+        1/((11-Speed)*80)
+    end.
+
+custom_relative_drag() ->
+    Speed = wings_pref:get_value(drag_speed_relative),
+    case wings_pref:get_value(drag_cam_dist_relative) of
+      true ->
+        #view{distance=D} = wings_view:current(),
+        D/((11-Speed)*((11-Speed)*900));
+      false ->
+        1/((11-Speed)*100)
+    end.
+
+custom_rotations_drag() ->
+    Speed = wings_pref:get_value(drag_speed_rotate),
+    1/((10.1-Speed)*8).
+
+default_drag(Units) ->
     #view{distance=D} = wings_view:current(),
     BasicSc = 1/500,
+    PercentSc = 1/500,
     DistSc = D/(9*500),
-    unit_scales_1(Units, BasicSc, DistSc).
+    AngleSc = 1/50,
+    unit_scales_1(Units, BasicSc, DistSc, PercentSc, AngleSc).
 
-unit_scales_1([U|Us], BasicSc, DistSc) ->
+unit_scales_1([U|Us], BasicSc, DistSc, PercentSc, AngleSc) ->
     Sc = case clean_unit(U) of
 	     distance -> DistSc;
 	     dx -> DistSc;
 	     dy -> DistSc;
 	     dz -> DistSc;
+	     angle -> AngleSc;
+	     percent -> PercentSc;
 	     _ -> BasicSc
 	 end,
-    [Sc|unit_scales_1(Us, BasicSc, DistSc)];
-unit_scales_1([], _, _) -> [].
+    [Sc|unit_scales_1(Us, BasicSc, DistSc, PercentSc, AngleSc)];
+unit_scales_1([], _, _, _, _) -> [].
 
 falloff([falloff|_]) -> 1.0;
 falloff([_|T]) -> falloff(T);
@@ -125,17 +169,6 @@ pad_offsets(Ds) ->
 	L -> Ds ++ lists:duplicate(4-L, 0.0)
     end.
 
-setup_offsets([O|Os], [U|Us]) ->
-    case clean_unit(U) of
-	angle ->
-	    [O/15.0E5/1.0E-5|setup_offsets(Os, Us)];
-	_Unit ->
-	    [O|setup_offsets(Os, Us)]
-    end;
-setup_offsets([O|Os], []) ->
-    [O|setup_offsets(Os, [])];
-setup_offsets([], []) -> [].
-
 %%
 %% Here we break apart the objects into two parts - static part
 %% (not moved during drag) and dynamic (part of objects actually
@@ -279,7 +312,7 @@ break_apart_general(Tvs) ->
 break_apart_general(#dlo{src_we=#we{id=Id}}=D, [{Id,Fun}|Tvs]) ->
     {D#dlo{drag={general,Fun}},Tvs};
 break_apart_general(D, Tvs) -> {D,Tvs}.
-    
+
 %%%
 %%% Handling of drag events.
 %%%
@@ -325,27 +358,58 @@ gl_rescale_normal() ->
 help_message(#drag{unit=Unit,mode_fun=ModeFun,mode_data=ModeData}) ->
     Accept = wings_msg:button_format(wings_s:accept()),
     ZMsg = zmove_help(Unit),
+    FpMsg = fpmove_help(Unit),
     Cancel = wings_msg:button_format([], [],wings_s:cancel()),
     NumEntry = ?__(1,"Numeric entry"),
     Tab = wings_util:key_format("[Tab]", NumEntry),
-    Msg = wings_msg:join([Accept,ZMsg,Cancel,Tab]),
+    Switch = switch(),
+    Constraint = ?__(2,"Switch Constraint Set")++Switch,
+    ShiftTab = wings_util:key_format("[Shift]+[Tab]", Constraint),
+    Msg = wings_msg:join([Accept,ZMsg,FpMsg,Cancel,Tab,ShiftTab]),
     MsgRight = ModeFun(help, ModeData),
     wings_wm:message(Msg, MsgRight).
 
+switch() ->
+    case wings_pref:get_value(con_alternate) of
+      true -> ?__(1," (using alternate constraints)");
+      false -> []
+    end.
+	
 zmove_help([_]) -> [];
 zmove_help([_,_]) -> [];
 zmove_help([_,_,falloff]) -> [];
 zmove_help([_,_,dz|_]) ->
     zmove_help_1(?__(1,"Drag to move along Z"));
+zmove_help([_,_,percent|_]) ->
+    zmove_help_1(?__(3,"Drag to adjust scale"));
+zmove_help([_,_,skip|_]) -> [];
 zmove_help([_,_,_|_]) ->
     zmove_help_1(?__(2,"Drag to adjust third parameter")).
 
+fpmove_help([_]) -> [];
+fpmove_help([_,_]) -> [];
+fpmove_help([_,_,_]) -> [];
+fpmove_help([_,_,_,falloff]) -> [];
+fpmove_help([_,_,_,_,falloff]) ->
+    fpmove_help_1(?__(1,"Drag to adjust forth parameter"));
+fpmove_help([_,_,_,angle|_]) ->
+    fpmove_help_1(?__(2,"Drag to adjust rotation"));
+fpmove_help([_,_,skip,Type|_]) ->
+    Str = wings_util:stringify(Type),
+    Help = ?__(4,"Drag to adjust") ++ " " ++ Str,
+    fpmove_help_1(Help);
+fpmove_help([_,_,_,_|_]) ->
+    fpmove_help_1(?__(1,"Drag to adjust forth parameter")).
+
 zmove_help_1(Msg) ->
     case wings_pref:get_value(camera_mode) of
 	tds -> wings_msg:mod_format(?CTRL_BITS, 3, Msg);
 	blender -> wings_msg:mod_format(?CTRL_BITS, 3, Msg);
+	sketchup -> wings_msg:mod_format(?CTRL_BITS, 3, Msg);
 	_ -> wings_msg:mod_format(0, 2, Msg)
     end.
+fpmove_help_1(Msg) ->
+	wings_msg:mod_format(0, 3, Msg).
 
 get_drag_event(Drag) ->
     case wings_pref:get_value(hide_sel_while_dragging) of
@@ -360,8 +424,15 @@ get_drag_event(Drag) ->
 get_drag_event_1(Drag) ->
     {replace,fun(Ev) -> handle_drag_event(Ev, Drag) end}.
 
-handle_drag_event(#keyboard{sym=9}, Drag) ->
-    numeric_input(Drag);
+handle_drag_event(#keyboard{sym=9, mod=Mod},Drag)->
+    case Mod band ?SHIFT_BITS =/= 0 of
+      true -> 
+        case wings_pref:get_value(con_alternate) of
+          true ->  wings_pref:set_value(con_alternate,false);
+          false -> wings_pref:set_value(con_alternate,true)
+        end, get_drag_event(Drag);
+      false -> numeric_input(Drag)
+    end;
 handle_drag_event(#mousebutton{button=2,state=?SDL_RELEASED},
 		  #drag{mmb_count=C}=Drag) when C > 2 ->
     get_drag_event_1(Drag#drag{mmb_count=0});
@@ -373,21 +444,25 @@ handle_drag_event(#mousebutton{button=3,state=?SDL_RELEASED,mod=Mod}=Ev,
 	true ->
 	    handle_drag_event_0(Ev, Drag)
     end;
+handle_drag_event(#mousebutton{button=3,state=?SDL_RELEASED},
+		  #drag{fp_count=C}=Drag) when C > 2 ->
+    get_drag_event_1(Drag#drag{fp_count=0});
+
 handle_drag_event(Event, Drag = #drag{st=St}) ->
     case wings_camera:event(Event, St, fun() -> redraw(Drag) end) of
 	next -> handle_drag_event_0(Event, Drag);
-	keep -> 
+	keep ->
 	    %% Clear any potential marker for an edge about to be
 	    %% cut (Cut RMB).
 	    wings_dl:map(fun(#dlo{hilite=none}=D, _) -> D;
 			    (D, _) -> D#dlo{hilite=none}
 			 end, []),
 	    %% Recalc unit_scales since zoom can have changed.
-	    #drag{xs=Xs0,ys=Ys0,zs=Zs0,unit=Unit,unit_sc=[US0|_]} = Drag,
+	    #drag{xs=Xs0,ys=Ys0,zs=Zs0,fp=Fp0,unit=Unit,unit_sc=[US0|_]} = Drag,
 	    US = [US1|_] = unit_scales(Unit),
-	    Adjust = US0/US1,	    
+	    Adjust = US0/US1,
 	    get_drag_event(Drag#drag{xs=Xs0*Adjust,ys=Ys0*Adjust,zs=Zs0*Adjust,
-				     unit_sc=US});
+		               fp=Fp0*Adjust,unit_sc=US});
 	Other ->
 	    %% Clear any potential marker for an edge about to be
 	    %% cut (Cut RMB).
@@ -405,11 +480,17 @@ handle_drag_event_0(#keyboard{unicode=C}=Ev,
 	    wings_wm:dirty(),
 	    wings_wm:message_right(ModeFun(help, ModeData)),
 	    Val = {ModeData,Drag0#drag.falloff},
-	    Drag1 = parameter_update(new_mode_data, Val,
-				     Drag0#drag{mode_data=ModeData}),
+	    Drag1 = case wings_pref:get_value(drag_resets) of
+	        false -> parameter_update(new_mode_data, Val,
+	                 Drag0#drag{mode_data=ModeData});
+	        true -> parameter_update(new_mode_data, Val,
+	                Drag0#drag{mode_data=ModeData,xs=0,ys=0})
+	    end,
 	    Drag = case ModeFun(units, ModeData) of
 		       none -> Drag1;
-		       Units -> Drag1#drag{unit=Units}
+		       Units ->
+		           UnitSc = unit_scales(Units),
+		           Drag1#drag{unit=Units,unit_sc=UnitSc}
 		   end,
 	    get_drag_event(Drag)
     end;
@@ -447,7 +528,9 @@ handle_drag_event_1(Event, #drag{st=St}=Drag0) ->
     case wings_hotkey:event(Event,St) of
 	next ->
 	    get_drag_event(Drag0);
-	{view,smoothed_preview} -> 
+	{view,quick_preview} ->
+	    get_drag_event(Drag0);
+	{view,smooth_proxy} ->
 	    get_drag_event(Drag0);
 	{view,Cmd} ->
 	    wings_view:command(Cmd, St),
@@ -478,12 +561,12 @@ ungrab(#drag{x=Ox,y=Oy}) ->
 invalidate_fun(#dlo{drag=none}=D, _) -> D;
 invalidate_fun(#dlo{src_we=We}=D, _) ->
     wings_draw:abort_split(D#dlo{src_we=We#we{es=none}}).
-    
+
 numeric_input(Drag0) ->
     {_,X,Y} = wings_wm:local_mouse_state(),
     Ev = #mousemotion{x=X,y=Y,state=0,mod=0},
     {Move0,Drag} = mouse_translate(Ev, Drag0),
-    wings_ask:dialog(?__(1,"Numeric Input"), 
+    wings_ask:dialog(?__(1,"Numeric Input"),
 		     make_query(Move0, Drag),
 		     fun(Res) ->
 			     {drag_arguments,make_move(Res, Drag)}
@@ -501,6 +584,8 @@ make_query_1([U0|Units], [V|Vals]) ->
 	    [{hframe,[{label,?__(2,"A")},
 		      {text,V,qrange(U0)},{label,[?DEGREE]}]}|
 	     make_query_1(Units, Vals)];
+	skip ->
+	     make_query_1(Units, Vals);
 	U ->
 	    [{hframe,[{label,qstr(U)},{text,V,qrange(U0)}]}|
 	     make_query_1(Units, Vals)]
@@ -512,8 +597,8 @@ qstr(dx) -> ?__(2,"Dx");
 qstr(dy) -> ?__(3,"Dy");
 qstr(dz) ->  ?__(4,"Dz");
 qstr(falloff) ->  ?__(5,"R");
-qstr(angle) ->  ?__(6,"A");
-qstr(Atom) -> atom_to_list(Atom).
+qstr(angle) ->	?__(6,"A");
+qstr(Atom) -> wings_util:stringify(Atom).
 
 qrange({_,{_,_}=Range}) -> [{range,Range}];
 qrange(_) -> [].
@@ -527,10 +612,12 @@ safe_mul_100(A) ->
 	{'EXIT',_} -> A;
 	P -> P
     end.
-    
+
 make_move(Move, #drag{unit=Units}) ->
     make_move_1(Units, Move).
 
+make_move_1([skip|Units],Vals) ->
+    [0|make_move_1(Units, Vals)];
 make_move_1([{percent,_}=Unit|Units], [V|Vals]) ->
     [clamp(Unit, V/100)|make_move_1(Units, Vals)];
 make_move_1([percent|Units], [V|Vals]) ->
@@ -555,9 +642,16 @@ view_changed(#drag{flags=Flags}=Drag0) ->
     case member(screen_relative, Flags) of
 	false -> Drag0;
 	true ->
+	    case member(keep_drag,Flags) of
+	      true ->
+	        wings_dl:map(fun view_changed_fun/2, []),
+	        {_,X,Y} = sdl_mouse:getMouseState(),
+	        Drag0#drag{x=X,y=Y};
+	      false ->
 	    wings_dl:map(fun view_changed_fun/2, []),
 	    {_,X,Y} = sdl_mouse:getMouseState(),
-	    Drag0#drag{x=X,y=Y,xs=0,ys=0,zs=0}
+	        Drag0#drag{x=X,y=Y,xs=0,ys=0,zs=0,fp=0}
+	    end
     end.
 
 view_changed_fun(#dlo{drag={matrix,Tr,_,_},transparent=#we{}=We}=D, _) ->
@@ -594,48 +688,52 @@ add_offset_1([D|Ds], [O|Ofs]) ->
     [D+O|add_offset_1(Ds, Ofs)];
 add_offset_1([], _) -> [].
 
-mouse_pre_translate(_, #mousemotion{state=Mask,mod=Mod}=Ev) ->
-    if
-	Mask band ?SDL_BUTTON_RMASK =/= 0,
-	Mod band ?CTRL_BITS =/= 0 ->
-	    {Ev#mousemotion{state=?SDL_BUTTON_MMASK},
-	     Mod band (bnot ?CTRL_BITS)};
-	true -> {Ev,Mod}
-    end.
+mouse_pre_translate(_, #mousemotion{mod=Mod}=Ev) -> {Ev,Mod}.
 
 mouse_range(#mousemotion{x=X0,y=Y0,state=Mask},
-	    #drag{x=OX,y=OY,xs=Xs0,ys=Ys0,zs=Zs0,
-		  xt=Xt0,yt=Yt0,mmb_count=Count0,
+	    #drag{x=OX,y=OY,xs=Xs0,ys=Ys0,zs=Zs0,fp=Fp0,
+		  xt=Xt0,yt=Yt0,mmb_count=Count0,fp_count=FpCount0,
 		  unit_sc=UnitScales,unit=Unit}=Drag,
 	    Mod) ->
     %%io:format("Mouse Range ~p ~p~n", [{X0,Y0}, {OX,OY,Xs0,Ys0}]),
     {X,Y} = wings_wm:local2global(X0, Y0),
     case wings_pref:lowpass(X- OX, Y-OY) of
 	{0,0} ->
-	    {mouse_scale([Xs0,-Ys0,-Zs0], UnitScales),
+	    {mouse_scale([Xs0,-Ys0,-Zs0,-Fp0], UnitScales),
 	     Drag#drag{xt=0,yt=0}};
 	{XD0,YD0} ->
 	    CS = constraints_scale(Unit,Mod,UnitScales),
 	    XD = CS*(XD0 + Xt0),
 	    YD = CS*(YD0 + Yt0),
-	    if
-		Mask band ?SDL_BUTTON_MMASK =/= 0 ->
+		case {Mask band ?SDL_BUTTON_MMASK =/= 0,Mask band ?SDL_BUTTON_RMASK =/= 0} of
+		  {true,false} ->
 		    Xs = Xs0,
 		    Ys = Ys0,
 		    Zs = case wings_pref:get_value(camera_mode) of
-			     maya -> Zs0 - XD;	%Horizontal motion
-			     _ -> Zs0 + YD	%Vertical motion
-			 end,
+		         maya -> Zs0 - XD;	%Horizontal motion
+		         _ -> Zs0 + YD	%Vertical motion
+		     end,
+		    Fp = Fp0,
+		    FpCount = FpCount0,
 		    Count = Count0 + 1;
-		true ->
+		  {false,true} ->
+		    Xs = Xs0,
+		    Ys = Ys0,
+		    Zs = Zs0,
+		    Fp = Fp0 + XD,
+		    FpCount = FpCount0 +1,
+		    Count = Count0;
+		  {_,_} ->
 		    Xs = Xs0 + XD,
 		    Ys = Ys0 + YD,
 		    Zs = Zs0,
+		    Fp = Fp0,
+		    FpCount = FpCount0,
 		    Count = Count0
 	    end,
 	    wings_io:warp(OX, OY),
-	    {mouse_scale([Xs,-Ys,-Zs], UnitScales),
-	     Drag#drag{xs=Xs,ys=Ys,zs=Zs,xt=XD0,yt=YD0,mmb_count=Count}}
+	    {mouse_scale([Xs,-Ys,-Zs,-Fp], UnitScales),
+	     Drag#drag{xs=Xs,ys=Ys,zs=Zs,fp=Fp,xt=XD0,yt=YD0,mmb_count=Count,fp_count=FpCount}}
     end.
 
 mouse_scale([D|Ds], [S|Ss]) ->
@@ -643,9 +741,17 @@ mouse_scale([D|Ds], [S|Ss]) ->
 mouse_scale(Ds, _) -> Ds.
 
 constraints_scale([U0|_],Mod,[UnitScales|_]) ->
-    case constraint_factor(clean_unit(U0),Mod) of
-	none -> 1.0;
-	{_,What} -> What*0.01/UnitScales
+    case wings_pref:get_value(con_alternate) of
+    true -> case constraint_factor_alt(clean_unit(U0),Mod) of
+    none -> 1.0;
+    {_,What} ->
+        What*0.01/UnitScales
+            end;
+    false -> case constraint_factor(clean_unit(U0),Mod) of
+            none -> 1.0;
+            {_,What} ->
+            What*0.01/UnitScales
+            end
     end.
 
 constrain(Ds0, Mod, #drag{unit=Unit}=Drag) ->
@@ -654,11 +760,18 @@ constrain(Ds0, Mod, #drag{unit=Unit}=Drag) ->
 
 constrain_0([U0|Us], [D0|Ds], Mod, Acc) ->
     U = clean_unit(U0),
-    D = case constraint_factor(U, Mod) of
+    D = case wings_pref:get_value(con_alternate) of
+      true -> case constraint_factor_alt(U, Mod) of
+        none -> D0;
+        {F1,F2} ->
+          round(D0*F1)*F2
+        end;
+      false -> case constraint_factor(U, Mod) of
 	    none -> D0;
-	    {F1,F2} -> 
-		round(D0*F1)*F2
-	end,
+	    {F1,F2} ->
+	      round(D0*F1)*F2
+            end
+    end,
     constrain_0(Us, Ds, Mod, [D|Acc]);
 constrain_0([_|_], [], _, Acc) -> reverse(Acc);
 constrain_0([], Ds, _, Acc) -> reverse(Acc, Ds).
@@ -674,21 +787,148 @@ clamp({_,{_Min,Max}}, D) when D > Max -> Max;
 clamp(_, D) -> D.
 
 constraint_factor(angle, Mod) ->
+    RCS = wings_pref:get_value(con_rot_shift),
+    RCC = wings_pref:get_value(con_rot_ctrl),
+    RCCS = wings_pref:get_value(con_rot_ctrl_shift),
+    RCA = wings_pref:get_value(con_rot_alt),
+    RCCA= wings_pref:get_value(con_rot_ctrl_alt),
+    RCSA = wings_pref:get_value(con_rot_shift_alt),
+    RCCSA = wings_pref:get_value(con_rot_ctrl_shift_alt),
     if
 	Mod band ?SHIFT_BITS =/= 0,
-	Mod band ?CTRL_BITS =/= 0 -> {15,1/15};%{150,1/15};
-	Mod band ?CTRL_BITS =/= 0 -> {1,1.0};%{15,1.0};
-	Mod band ?SHIFT_BITS =/= 0 ->{1/15,15.0};%{1,15.0};
-	%%true -> {1.0E5,1.0E-5}
+	Mod band ?ALT_BITS =/= 0,
+	Mod band ?CTRL_BITS =/= 0 -> {1/RCCSA,RCCSA};
+	Mod band ?SHIFT_BITS =/= 0,
+	Mod band ?ALT_BITS =/= 0 -> {1/RCSA,RCSA};
+	Mod band ?CTRL_BITS =/= 0,
+	Mod band ?ALT_BITS =/= 0 -> {1/RCCA,RCCA};
+	Mod band ?SHIFT_BITS =/= 0,
+	Mod band ?CTRL_BITS =/= 0 -> {1/RCCS,RCCS};
+	Mod band ?CTRL_BITS =/= 0 -> {1/RCC,RCC};
+	Mod band ?SHIFT_BITS =/= 0 -> {1/RCS,RCS};
+	Mod band ?ALT_BITS =/= 0 -> {1/RCA,RCA};
 	true -> none
     end;
-constraint_factor(_, Mod) ->
+constraint_factor(percent, Mod) ->
+    SCS = wings_pref:get_value(con_scale_shift),
+    SCC = wings_pref:get_value(con_scale_ctrl),
+    SCCS = wings_pref:get_value(con_scale_ctrl_shift),
+    SCA = wings_pref:get_value(con_scale_alt),
+    SCCA = wings_pref:get_value(con_scale_ctrl_alt),
+    SCSA = wings_pref:get_value(con_scale_shift_alt),
+    SCCSA = wings_pref:get_value(con_scale_ctrl_shift_alt),
     if
-	Mod band ?SHIFT_BITS =/= 0,
-	Mod band ?CTRL_BITS =/= 0 -> {100,1/100};
-	Mod band ?CTRL_BITS =/= 0 -> {10,1/10};
-	Mod band ?SHIFT_BITS =/= 0 -> {1,1.0};
-	true -> none
+      Mod band ?SHIFT_BITS =/= 0,
+      Mod band ?ALT_BITS =/= 0,
+      Mod band ?CTRL_BITS =/= 0-> {1/SCCSA,SCCSA};
+      Mod band ?SHIFT_BITS =/= 0,
+      Mod band ?ALT_BITS =/= 0 -> {1/SCSA,SCSA};
+      Mod band ?CTRL_BITS =/= 0,
+      Mod band ?ALT_BITS =/= 0 -> {1/SCCA,SCCA};
+      Mod band ?CTRL_BITS =/= 0,
+      Mod band ?SHIFT_BITS =/= 0 -> {1/SCCS,SCCS};
+      Mod band ?CTRL_BITS =/= 0 -> {1/SCC,SCC};
+      Mod band ?SHIFT_BITS =/= 0 -> {1/SCS,SCS};
+      Mod band ?ALT_BITS =/= 0 -> {1/SCA,SCA};
+      true -> none
+    end;
+constraint_factor(_, Mod) ->
+    DCS = wings_pref:get_value(con_dist_shift),
+    DCC = wings_pref:get_value(con_dist_ctrl),
+    DCCS = wings_pref:get_value(con_dist_ctrl_shift),
+    DCA = wings_pref:get_value(con_dist_alt),
+    DCCA = wings_pref:get_value(con_dist_ctrl_alt),
+    DCSA = wings_pref:get_value(con_dist_shift_alt),
+    DCCSA = wings_pref:get_value(con_dist_ctrl_shift_alt),
+    if
+      Mod band ?SHIFT_BITS =/= 0,
+      Mod band ?ALT_BITS =/= 0,
+      Mod band ?CTRL_BITS =/= 0-> {1/DCCSA,DCCSA};
+      Mod band ?SHIFT_BITS =/= 0,
+      Mod band ?ALT_BITS =/= 0 -> {1/DCSA,DCSA};
+      Mod band ?CTRL_BITS =/= 0,
+      Mod band ?ALT_BITS =/= 0 -> {1/DCCA,DCCA};
+      Mod band ?CTRL_BITS =/= 0,
+      Mod band ?SHIFT_BITS =/= 0 -> {1/DCCS,DCCS};
+      Mod band ?CTRL_BITS =/= 0 -> {1/DCC,DCC};
+      Mod band ?SHIFT_BITS =/= 0 -> {1/DCS,DCS};
+      Mod band ?ALT_BITS =/= 0 -> {1/DCA,DCA};
+      true -> none
+    end.
+constraint_factor_alt(angle, Mod) ->
+    RCRS = filter_angle(wings_pref:get_value(con_rot_shift)),
+    RCRC = filter_angle(wings_pref:get_value(con_rot_ctrl)),
+    RCRCS = filter_angle(wings_pref:get_value(con_rot_ctrl_shift)),
+    RCRA = filter_angle(wings_pref:get_value(con_rot_alt)),
+    RCRCA = filter_angle(wings_pref:get_value(con_rot_ctrl_alt)),
+    RCRSA = filter_angle(wings_pref:get_value(con_rot_shift_alt)),
+    RCRCSA = filter_angle(wings_pref:get_value(con_rot_ctrl_shift_alt)),
+    if
+      Mod band ?SHIFT_BITS =/= 0,
+      Mod band ?ALT_BITS =/= 0,
+      Mod band ?CTRL_BITS =/= 0 -> {1/RCRCSA,RCRCSA};
+      Mod band ?SHIFT_BITS =/= 0,
+      Mod band ?ALT_BITS =/= 0 -> {1/RCRSA,RCRSA};
+      Mod band ?CTRL_BITS =/= 0,
+      Mod band ?ALT_BITS =/= 0 -> {1/RCRCA,RCRCA};
+      Mod band ?SHIFT_BITS =/= 0,
+      Mod band ?CTRL_BITS =/= 0 -> {1/RCRCS,RCRCS};
+      Mod band ?CTRL_BITS =/= 0 -> {1/RCRC,RCRC};
+      Mod band ?SHIFT_BITS =/= 0 -> {1/RCRS,RCRS};
+      Mod band ?ALT_BITS =/= 0 -> {1/RCRA,RCRA};
+      true -> none
+    end;
+constraint_factor_alt(percent, Mod) ->
+    SCS = 1.0/wings_pref:get_value(con_scale_shift),
+    SCC = 1.0/wings_pref:get_value(con_scale_ctrl),
+    SCCS = 1.0/wings_pref:get_value(con_scale_ctrl_shift),
+    SCA = 1.0/wings_pref:get_value(con_scale_alt),
+    SCCA = 1.0/wings_pref:get_value(con_scale_ctrl_alt),
+    SCSA = 1.0/wings_pref:get_value(con_scale_shift_alt),
+    SCCSA = 1.0/wings_pref:get_value(con_scale_ctrl_shift_alt),
+    if
+      Mod band ?SHIFT_BITS =/= 0,
+      Mod band ?ALT_BITS =/= 0,
+      Mod band ?CTRL_BITS =/= 0-> {1/SCCSA,SCCSA};
+      Mod band ?SHIFT_BITS =/= 0,
+      Mod band ?ALT_BITS =/= 0 -> {1/SCSA,SCSA};
+      Mod band ?CTRL_BITS =/= 0,
+      Mod band ?ALT_BITS =/= 0 -> {1/SCCA,SCCA};
+      Mod band ?CTRL_BITS =/= 0,
+      Mod band ?SHIFT_BITS =/= 0 -> {1/SCCS,SCCS};
+      Mod band ?CTRL_BITS =/= 0 -> {1/SCC,SCC};
+      Mod band ?SHIFT_BITS =/= 0 -> {1/SCS,SCS};
+      Mod band ?ALT_BITS =/= 0 -> {1/SCA,SCA};
+      true -> none
+    end;
+constraint_factor_alt(_, Mod) ->
+    DCS = wings_pref:get_value(con_dist_a_shift),
+    DCC = wings_pref:get_value(con_dist_a_ctrl),
+    DCCS = wings_pref:get_value(con_dist_a_ctrl_shift),
+    DCA = wings_pref:get_value(con_dist_a_alt),
+    DCCA = wings_pref:get_value(con_dist_a_ctrl_alt),
+    DCSA = wings_pref:get_value(con_dist_a_shift_alt),
+    DCCSA = wings_pref:get_value(con_dist_a_ctrl_shift_alt),
+    if
+      Mod band ?SHIFT_BITS =/= 0,
+      Mod band ?ALT_BITS =/= 0,
+      Mod band ?CTRL_BITS =/= 0-> {1/DCCSA,DCCSA};
+      Mod band ?SHIFT_BITS =/= 0,
+      Mod band ?ALT_BITS =/= 0 -> {1/DCSA,DCSA};
+      Mod band ?CTRL_BITS =/= 0,
+      Mod band ?ALT_BITS =/= 0 -> {1/DCCA,DCCA};
+      Mod band ?CTRL_BITS =/= 0,
+      Mod band ?SHIFT_BITS =/= 0 -> {1/DCCS,DCCS};
+      Mod band ?CTRL_BITS =/= 0 -> {1/DCC,DCC};
+      Mod band ?SHIFT_BITS =/= 0 -> {1/DCS,DCS};
+      Mod band ?ALT_BITS =/= 0 -> {1/DCA,DCA};
+      true -> none
+    end.
+
+filter_angle(Degrees) ->
+    case Degrees =/= 180.0 of
+      true -> 180.0 - Degrees;
+      false -> 180.0
     end.
 
 %%%
@@ -720,7 +960,7 @@ possible_falloff_update(_, #drag{falloff=none}=Drag) -> Drag;
 possible_falloff_update(Move, Drag) ->
     NewFalloff = lists:last(Move),
     parameter_update(new_falloff, NewFalloff, Drag#drag{falloff=NewFalloff}).
-    
+
 parameter_update(Key, Val, Drag) ->
     wings_dl:map(fun(D, _) ->
 			 parameter_update_fun(D, Key, Val)
@@ -736,7 +976,7 @@ parameter_update_fun(D, _, _) -> D.
 
 translate({Xt0,Yt0,Zt0}, Dx, VsPos, Acc) ->
     Xt = Xt0*Dx, Yt = Yt0*Dx, Zt = Zt0*Dx,
-    foldl(fun({V,{X,Y,Z}}, A) -> 
+    foldl(fun({V,{X,Y,Z}}, A) ->
 		  Pos = wings_util:share(X+Xt, Y+Yt, Z+Zt),
 		  [{V,Pos}|A]
 	  end, Acc, VsPos).
@@ -747,7 +987,7 @@ progress_units([], []) -> [].
 
 clean_unit({Unit,_}) when is_atom(Unit) -> Unit;
 clean_unit(Unit) when is_atom(Unit) -> Unit.
-    
+
 unit(angle, A) ->
     trim(io_lib:format("~10.2f~c  ", [A,?DEGREE]));
 unit(number, N) ->
@@ -764,9 +1004,11 @@ unit(percent, P) ->
     trim(io_lib:format("~.2f%  ", [P*100.0]));
 unit(falloff, R) ->
     ["R: "|trim(io_lib:format("~10.2f", [R]))];
-unit(Unit, Move) ->
-    io:format("~p\n", [{Unit,Move}]),
-    [].
+unit(skip,_) ->
+    %% the atom 'skip' can be used as a place holder. See wpc_arc.erl
+    [];
+unit(Unit, D) ->
+    [wings_util:stringify(Unit) ++ ": "|trim(io_lib:format("~10.4f  ", [D]))].
 
 trim([$\s|T]) -> trim(T);
 trim([[_|_]=H|T]) ->
@@ -775,7 +1017,7 @@ trim([[_|_]=H|T]) ->
 	S -> [S|T]
     end;
 trim(S) -> S.
-    
+
 normalize(Move, #drag{mode_fun=ModeFun,mode_data=ModeData,
 		      st=#st{shapes=Shs0}=St}) ->
     ModeFun(done, ModeData),
