@@ -3,12 +3,12 @@
 %%
 %%     Implementation of pulldown and popup menus.
 %%
-%%  Copyright (c) 2001-2004 Bjorn Gustavsson
+%%  Copyright (c) 2001-2008 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_menu.erl,v 1.132 2005/06/27 20:13:06 bjorng Exp $
+%%     $Id: wings_menu.erl 389 2008-10-02 05:22:31Z optigon $
 %%
 
 -module(wings_menu).
@@ -31,7 +31,7 @@
 	 w,					%Width of menu (pixels)
 	 h,					%Height of menu (pixels)
 	 hs,					%Tuple: height of each entry.
-	 sel=none,				%Selected item (1..size(Menu))
+	 sel=none,				%Selected item (1..tuple_size(Menu))
 	 sel_side=left,				%Selection on left or right.
 	 ns=[],					%Name stack.
 	 menu,					%Original menu term
@@ -257,7 +257,7 @@ norm_help(Elem, true) ->  norm_help_adv(Elem, {[],[],[]}).
 
 norm_help_basic({_,_,_,Help,_}=Item) when is_list(Help) ->
     Item;
-norm_help_basic({T,N,Hot,Help,Ps}) when is_tuple(Help), size(Help) > 0 ->
+norm_help_basic({T,N,Hot,Help,Ps}) when tuple_size(Help) > 0 ->
     {T,N,Hot,element(1, Help),Ps}.
 
 norm_help_adv({T,N,Hot,[],Ps}, Empty)  ->
@@ -279,7 +279,7 @@ norm_help_adv({T,N,Hot,{Hl,Hm,[]},Ps}=Item, _) ->
     end.
 
 menu_dims(Menu) ->
-    menu_dims(Menu, size(Menu), 0, 0, 0, []).
+    menu_dims(Menu, tuple_size(Menu), 0, 0, 0, []).
 
 menu_dims(_Menu, 0, MaxA, MaxB, MaxC, H) -> {MaxA,MaxB,MaxC,H};
 menu_dims(Menu, I, MaxA0, MaxB0, MaxC0, Hacc) ->
@@ -431,20 +431,23 @@ handle_key_1(cancel, _) ->
 handle_key_1(delete, Mi0) ->
     %% Delete hotkey bound to this entry.
     case current_command(Mi0) of
-	none -> keep;
-	Cmd ->
-	    wings_wm:dirty(),
-	    NextKey = wings_hotkey:delete_by_command(Cmd),
-	    Mi = set_hotkey(NextKey, Mi0),
-	    get_menu_event(Mi)
+      [] -> keep;
+      [_|_]=Cmds0 ->
+        Cmds = case Cmds0 of
+          [{1,{A,{B,false}}}] -> [{A,{B,false}},{A,{B,true}}];
+          [{1,{A,{B,{C,false}}}}] -> [{A,{B,{C,false}}},{A,{B,{C,true}}}];
+          _Cmd -> [C || {_,C} <- Cmds0]
+        end,
+        case wings_hotkey:hotkeys_by_commands(Cmds) of
+          [] -> keep;			%No hotkeys for this entry.
+          Hotkeys -> hotkey_delete_dialog(Hotkeys)
+        end
     end;
 handle_key_1(insert, Mi) ->
     %% Define new hotkey for this entry.
     case current_command(Mi) of
-	none -> keep;
-	Cmd ->
-	    wings_wm:dirty(),
-	    get_hotkey(Cmd, Mi)
+	[] -> keep;
+	[_|_]=Cmds -> get_hotkey(Cmds, Mi)
     end;
 handle_key_1(_, _) -> keep.
 
@@ -455,44 +458,65 @@ key(#keyboard{sym=?SDLK_DELETE}) -> delete;
 key(#keyboard{unicode=$\\}) -> delete;
 key(_) -> none.
 
-current_command(Mi) ->
-    case current_command_1(Mi) of
-	none -> none;
-	{Cmd0,OptionBox} ->
-	    Cmd = add_option(OptionBox, Cmd0, false),
-	    simplify_command(Cmd)
-    end.
-
-current_command_1(#mi{sel=none}) -> none;
-current_command_1(#mi{sel=Sel,menu=Menu,ns=Names,owner=Owner})
-  when Owner == geom; element(1, Owner) == geom ->
+current_command(#mi{sel=none}) -> [];
+current_command(#mi{sel=Sel,menu=Menu,ns=Names,owner=Owner}=Mi)
+  when Owner =:= geom; element(1, Owner) =:= geom ->
     case element(Sel, Menu) of
 	{_,Name,_,_,Ps} when is_atom(Name); is_integer(Name) ->
-	    {build_command(Name, Names),have_option_box(Ps)};
-	{_,Fun,_,_,Ps} when is_function(Fun) ->
-	    Cmd = Fun(1, Names),
-	    case is_ascii_clean(Cmd) of
-		true -> {Cmd,have_option_box(Ps)};
-		false -> none
-	    end;
-	{_,{Name,Fun},_,_,Ps} when is_function(Fun) ->
-	    Cmd = Fun(1, [Name|Names]),
-	    case is_tuple(Cmd) andalso is_ascii_clean(Cmd) of
-		true -> {Cmd,have_option_box(Ps)};
-		false -> none
-	    end;
+	    Cmd0 = build_command(Name, Names),
+	    OptionBox = have_option_box(Ps),
+	    Cmd = add_option(OptionBox, Cmd0, false),
+	    [{1,simplify_command(Cmd)}];
+	{_,Fun,_,_,Ps} when is_function(Fun), is_function(Fun, 2) ->
+	    Try = fun(B) ->
+			  Cmd0 = Fun(B, Names),
+			  case Cmd0 =/= ignore andalso is_ascii_clean(Cmd0) of
+			      true ->
+				  OptionBox = have_option_box(Ps),
+				  Cmd = add_option(OptionBox, Cmd0, false),
+				  simplify_command(Cmd);
+			      false -> none
+			  end
+		  end,
+	    all_current_commands(Try, Mi);
+	{_,{Name,Fun},_,_,Ps} when is_function(Fun), is_function(Fun, 2) ->
+	    Try = fun(B) ->
+			  Cmd0 = Fun(B, [Name|Names]),
+			  case is_tuple(Cmd0) andalso is_ascii_clean(Cmd0) of
+			      true ->
+				  OptionBox = have_option_box(Ps),
+				  Cmd = add_option(OptionBox, Cmd0, false),
+				  simplify_command(Cmd);
+			      false -> none
+			  end
+		  end,
+	    all_current_commands(Try, Mi);
 	_Other ->
 	    io:format("~p\n", [_Other]),
-	    none
+	    []
     end;
-current_command_1(_) -> none.
+current_command(_) -> [].
+
+all_current_commands(Fun, #mi{adv=false}) ->
+    %% This menu is in basic mode, so we must only return the
+    %% command for LMB.
+    all_current_commands_1([1], Fun);
+all_current_commands(Fun, #mi{adv=true}) ->
+    all_current_commands_1([1,2,3], Fun).
+
+all_current_commands_1([B|Bs], Fun) ->
+    case Fun(B) of
+	none -> all_current_commands_1(Bs, Fun);
+	Cmd -> [{B,Cmd}|all_current_commands_1(Bs, Fun)]
+    end;
+all_current_commands_1([], _) -> [].
 
 %% Test if a term can be represented in a text file and read back.
 is_ascii_clean([H|T]) ->
     is_ascii_clean(H) andalso is_ascii_clean(T);
 is_ascii_clean([]) -> true;
 is_ascii_clean(T) when is_tuple(T) ->
-    is_tuple_ascii_clean(1, size(T), T);
+    is_tuple_ascii_clean(1, tuple_size(T), T);
 is_ascii_clean(Num) when is_number(Num) -> true;
 is_ascii_clean(Atom) when is_atom(Atom) -> true;
 is_ascii_clean(_) -> false.
@@ -689,7 +713,7 @@ selected_item(Y, #mi{adv=Adv,ymarg=Margin,h=H,menu=Menu}=Mi) ->
 	    %% Below the lower margin. Pretend that the cursor is over the
 	    %% last row (we assume that it cannot be inactive text
 	    %% or a separator).
-	    size(Menu)
+	    tuple_size(Menu)
     end.
 
 selected_item_1(Y, #mi{hs=Hs}=Mi) ->
@@ -775,14 +799,17 @@ menu_draw_1(Y, Ps, Sel, #mi{sel=Sel,sel_side=Side,w=W},
 	    DrawLeft, DrawRight) ->
     %% Draw blue background for highlighted item.
     wings_io:set_color(wings_pref:get_value(menu_hilite)),
+    Color = wings_pref:get_value(menu_hilite),
     Cw = wings_text:width(),
     Right = W - (2*right_width(Ps)) - Cw,
     case Side of
 	right ->
-	    gl:recti(Right, Y-?CHAR_HEIGHT, Right+3*Cw-2, Y+3),
+	    {X1,Y1,X2,Y2} = {Right, Y-?CHAR_HEIGHT, Right+3*Cw-2, Y+3},
+	    wings_io:gradient_rect(X1, Y1, X2-X1, Y2-Y1, Color),
 	    wings_io:set_color(wings_pref:get_value(menu_text));
 	left ->
-	    gl:recti(?CHAR_WIDTH, Y-?CHAR_HEIGHT, Right, Y+3),
+	    {X1,Y1,X2,Y2} = {?CHAR_WIDTH, Y-?CHAR_HEIGHT, Right, Y+3},
+	    wings_io:gradient_rect(X1, Y1, X2-X1, Y2-Y1, Color),
 	    wings_io:set_color(wings_pref:get_value(menu_hilited_text))
     end,
     DrawLeft(),
@@ -892,10 +919,6 @@ draw_right_1(X0, Y0, Mw, Ps) ->
 	    wings_io:border(X, Y, Cw, Ch-1, Color)
     end.
 
-draw_submenu(_Adv, Item, _X, _Y) when is_atom(Item);
-				      is_integer(Item);
-				      is_list(Item) ->
-    ok;
 draw_submenu(true, _Item, _X, _Y) -> ok;
 draw_submenu(false, _Item, X, Y) ->
     Cw = wings_text:width(),
@@ -960,13 +983,39 @@ have_magnet(Ps) ->
 %%% Get a key to bind a command to.
 %%%
 
-get_hotkey(Cmd, Mi) ->
+get_hotkey([{1,Cmd}], Mi) ->
     wings_wm:dirty(),
-    wings_wm:message(?__(1,"Press key to bind command to.")),
+    case Cmd of
+      {A,{B,false}} -> get_hotkey([{1,Cmd},{3,{A,{B,true}}}],Mi);
+      {A,{B,{C,false}}} -> get_hotkey([{1,Cmd},{3,{A,{B,{C,true}}}}],Mi);
+      Cmd ->
+        wings_wm:message(hotkey_key_message(Cmd)),
+        {push,fun(Ev) ->
+          handle_key_event(Ev, Cmd, Mi)
+          end}
+    end;
+get_hotkey([_|_]=Cmds, Mi) ->
+    wings_wm:dirty(),
+    wings_wm:message(hotkey_mouse_message(Cmds)),
     {push,fun(Ev) ->
-		  handle_key_event(Ev, Cmd, Mi)
+		  handle_button_event(Ev, Cmds, Mi)
 	  end}.
 
+handle_button_event(redraw, _Cmds, Mi) ->
+    redraw(Mi),
+    keep;
+handle_button_event(#mousebutton{button=B,state=?SDL_RELEASED}, Cmds, Mi) ->
+    case keysearch(B, 1, Cmds) of
+	{value,{B,Cmd}} ->
+	    wings_wm:message(hotkey_key_message(Cmd)),
+	    {replace,fun(Ev) ->
+			     handle_key_event(Ev, Cmd, Mi)
+		     end};
+	false ->
+	    keep
+    end;
+handle_button_event(_, _, _) ->keep.
+
 handle_key_event(redraw, _Cmd, Mi) ->
     redraw(Mi),
     keep;
@@ -990,3 +1039,41 @@ do_bind(Win, Ev, Cmd, Mi0) ->
     Mi = set_hotkey(Keyname, Mi0),
     wings_wm:send(Win, Mi),
     ignore.
+
+hotkey_mouse_message(Cmds) ->
+    {Lmb,Mmb,Rmb} = hotkey_mouse_message_1(Cmds, [], [], []),
+    [?__(1,"Click a mouse button to choose command to bind"),
+     ":   ",wings_msg:button_format(Lmb, Mmb, Rmb)].
+
+hotkey_mouse_message_1([{1,Cmd}|T], _L, M, R) ->
+    hotkey_mouse_message_1(T, wings_util:stringify(Cmd), M, R);
+hotkey_mouse_message_1([{2,Cmd}|T], L, _M, R) ->
+    hotkey_mouse_message_1(T, L, wings_util:stringify(Cmd), R);
+hotkey_mouse_message_1([{3,Cmd}|T], L, M, _R) ->
+    hotkey_mouse_message_1(T, L, M, wings_util:stringify(Cmd));
+hotkey_mouse_message_1([], L, M, R) -> {L,M,R}.
+
+hotkey_key_message(Cmd) ->
+    [?__(1,"Press the key to bind the \""),
+     wings_util:stringify(Cmd),
+     ?__(2,"\" command to.")].
+
+%%%
+%%% Hotkey deletion dialog.
+%%%
+
+hotkey_delete_dialog(Hotkeys) ->
+    Fun = fun(Res) ->
+		  [wings_hotkey:unbind(K) || {K,true} <- Res],
+		  ignore
+	  end,
+    Dialog = mk_dialog(Hotkeys),
+    wings_ask:dialog(?__(1,"Delete Hotkeys"), Dialog, Fun).
+
+mk_dialog([{Key,Keyname,Cmd,Src}|T]) ->
+    [mk_key_item(Key, Keyname, Cmd, Src)|mk_dialog(T)];
+mk_dialog([]) ->
+    [separator,{label,?__(1,"Check all hotkeys to be deleted.")}].
+
+mk_key_item(Key, Keyname, Cmd, _Src) ->
+    {Keyname ++ ": " ++ Cmd,false,[{key,Key}]}.
